/* filename:    nkpm_pri2.g
** description: The program deletes the draws that lie outside of the parameter space
** created:     
*/

/********************************************************
* House cleaning and set defaults
********************************************************/
//new;
closeall;
//library pgraph, user;
cls;
outwidth 256;
format /mat /on /mb1 /ros 16,8;

#include nkmp_mod.src;
#include nkmp_para.src;
#include nkmp_spec.g;

#include pathspec.g;

para  = pmsv(msel);
npara = rows(para);

block_size = 1000;
mhrun      = "0";   /* NOTE: we use mhrun = 0 to denote prior */

lpath      = parapath $+ path_sep $+ "mhrun" $+ mhrun;  
opath      = parapath $+ path_sep $+ "mhrun" $+ mhrun;

/* Open files with Parameter Draws
*/

ltheta = lpath $+ path_sep $+ "pa0";
open fhltheta = ^ltheta for read;

/* Initialize Output files
*/
otheta = opath  $+ path_sep $+ "pa";
create fhotheta = ^otheta with THETA, npara, 8;

odet  = opath  $+ path_sep $+ "dt";
create fhdet = ^odet with DET, 2, 2;

/* Initialization
*/ 
eofloop = 0;
loopct  = 0;
nread   = 0;
nvalid  = 0;

do until eofloop; 

   /* Read blocks of size block_size
   */
   thetasim = readr( fhltheta, block_size );

   nread   = nread + rows(thetasim);

   /* Eliminate draws that are in the non-existence and indeterminacy region
   */
   invaliddraw = zeros(rows(thetasim),1);
   detsim      = zeros(rows(thetasim),2);

   j = 1;
   do until j > rows(thetasim);

      /* For each element of the block, compute the desired statistic
      */
      {TT,QQ,RR,HH,DD,ZZ,VV,RC}=sysmat(thetasim[j,.]',msel);
      
      if (RC[1] == 1) AND (RC[2]==0);
         /* indeterminacy */
         detsim[j,2] = 1;
         
      elseif (RC[1] == 1) AND (RC[2]==1); 
         /* determinacy */
         detsim[j,1] = 1; 
       
      else;
         invaliddraw[j,1] = 1;  

      endif; 

      if msel[1,1] == 0;
         /* check for validity of correlation parameter
         */
         if (thetasim[j,34] GE 1) OR (thetasim[j,34] LE -1);
            invaliddraw[j,1] = 1;
         endif;
      endif;
      
      j = j+1;
  endo;

  thetasim = delif(thetasim, invaliddraw);
  detsim   = delif(detsim, invaliddraw);  
  thetasim = delif(thetasim, detsim[.,2]);
  
  nvalid  = nvalid + rows(thetasim);

  /* Write Output
  */
  wr = writer(fhdet,detsim);
  wr = writer(fhotheta,thetasim);
  
  loopct = loopct + 1;

  "Loop            " loopct;
  "Relative Size of Regions [det, indet]"; meanc(detsim)';
  " ";pause(1e-20);
  eofloop = eof(fhltheta);
  
endo;

"Fraction of valid draws" nvalid/nread;

closeall fhdet, fhltheta, fhotheta;
